import parseResponse from '../response';

describe('Testing 2xx Error...', function() {
  it('should throw authentication error', function() {
    const res = {
      status: 200,
      headers: {'content-type': 'text/html'},
      body: `
      <html>
        <body>
          <form action="/portalserver/j_spring_security_check" method="POST"></form>
        </body>
      </html>`,
    };

    expect(() => parseResponse({}, {}, res))
      .toThrowError(/HTML Error: Invalid Session/);
  });

  it('should throw 200 html printed error', function() {
    const res = {
      status: 200,
      headers: {'content-type': 'text/html'},
      body: `
      <html>
        <body>
          <div class="bd-errorMsg">
            <div>
              <p></p>
              <p foo="bar">Test Error Response</p>
            </div>
          </div>
        </body>
      </html>`,
    };
    
    expect(() => parseResponse({}, {}, res))
      .toThrowError(/HTML Error: Test Error Response/g);
  });
  
  it('should throw 200 html error', function() {
    const res = {
      status: 200,
      headers: {'content-type': 'application/xml'},
      body: `<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
        <import>
          <level>ERROR</level>
          <message>Exception Test</message>
        </import>
      `,
    };
    
    expect(() => parseResponse({}, {}, res))
      .toThrowError(/HTML Error: Exception Test/);
  });
  it('should return proper response', function() {
    const res = {
      status: 200,
      headers: {'content-type': 'application/xml'},
      body: `<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
        <proper>response</proper>
      `,
    };
    expect(parseResponse({}, {}, res))
      .toMatch(/<proper>response/);
  });
});
